package com.webchat.connect.service;


import com.webchat.common.bean.APIResponseBean;
import com.webchat.common.bean.APIResponseBeanUtil;
import com.webchat.common.util.JsonUtil;
import com.webchat.domain.vo.response.UserBaseResponseInfoVO;
import com.webchat.rmi.user.UserServiceClient;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Set;

@Slf4j
@Service
public class AccountService {

    @Autowired
    private UserServiceClient userServiceClient;

    /**
     * 获取账号详情，底层走Redis查询
     * @param account
     * @return
     */
    public UserBaseResponseInfoVO accountInfo(String account) {
        if (StringUtils.isBlank(account)) {
            return null;
        }
        APIResponseBean<UserBaseResponseInfoVO> responseBean = userServiceClient.userInfo(account);
        if (APIResponseBeanUtil.isOk(responseBean)) {
            return responseBean.getData();
        }
        log.error("RPC Invoke Error ====> UserServiceClient.userInfo error, account:{}, responseBean:{}",
                   account, JsonUtil.toJsonString(responseBean));
        return null;
    }

    /**
     * 获取群组下的群成员用户id集合
     *
     * @param groupAccount
     * @return
     */
    public Set<String> getGroupUserIds(String groupAccount) {

        APIResponseBean<Set<String>> responseBean = userServiceClient.getGroupUserIds(groupAccount);
        if (APIResponseBeanUtil.isOk(responseBean)) {
            return responseBean.getData();
        }
        return null;
    }
}
